<?php

namespace WpieApp\Core\Office\Writer\CSV;

use WP_Error;
use WpieApp\Core\Helpers\Helper;

defined( 'WPIE__FILE__' ) || exit;


class Writer {

        
        private $fileHandle = null;

        
        private $delimiter = ",";

        
        private $enclosure = "\"";

        
        private $options = [];

        
        public function load( $file = "", $options = [] ) {

                if ( !Helper::isNonEmptyStr( $file ) ) {
                         /* translators: %s: File path */
                        return new WP_Error( 'wpieError', sprintf( __( "Invalid File URL : %s", 'wp-import-export' ), $file ) );
                }

                if ( is_resource( $this->fileHandle ) ) {
                        return true;
                }

                $this->options = $options;

                $dir = pathinfo( $file, PATHINFO_DIRNAME );

                Helper::createDir( $dir );

                $this->fileHandle = fopen( $file, 'a' );

                if ( $this->fileHandle === false ) {
                         /* translators: %s: File path */
                        return new WP_Error( 'wpieError', sprintf( __( "File can't Open : %s", 'wp-import-export' ), $file ) );
                }

                $this->addBom( $file );

                $this->setDelimiter();

                $this->setEnclosure();

                return true;
        }

        
        private function addBom( $file = "" ) {

                if ( filesize( $file ) > 0 ) {
                        return;
                }

                fwrite( $this->fileHandle, chr( 0xEF ) . chr( 0xBB ) . chr( 0xBF ) );
        }

        
        private function setDelimiter() {

                $delimiter = isset( $this->options[ 'delimiter' ] ) && Helper::isNonEmptyStr( $this->options[ 'delimiter' ] ) ? trim( $this->options[ 'delimiter' ] ) : ',';

                if ( !Helper::isNonEmptyStr( $delimiter ) || "," === $delimiter ) {
                        $this->delimiter = $delimiter;
                        return;
                }

                if ( Helper::contains( $delimiter, ";" ) ) {
                        $detectDelim = ';';
                } elseif ( Helper::contains( $delimiter, "|" ) ) {
                        $detectDelim = '|';
                } elseif ( Helper::contains( $delimiter, "t" ) ) {
                        $detectDelim = "\t";
                } else {
                        $detectDelim = $delimiter;
                }


                $this->delimiter = $detectDelim;
        }

        
        private function setEnclosure() {

                $enclosure = isset( $this->options[ 'enclosure' ] ) && Helper::isNonEmptyStr( $this->options[ 'enclosure' ] ) ? trim( $this->options[ 'enclosure' ] ) : '';

                if ( !Helper::isNonEmptyStr( $enclosure ) ) {
                        return;
                }

                $this->enclosure = ("'" === $enclosure ? "'" : '"');
        }

        
        public function addRow( $row = [] ) {

                if ( !is_resource( $this->fileHandle ) ) {
                        return false;
                }

                $row = array_map( [ $this, 'sanitize' ], $row );

                if ( $this->isEmptyRow( $row ) ) {
                        return false;
                }

                fputcsv( $this->fileHandle, $row, $this->delimiter );

                return true;
        }

        
        private function sanitize( $data = "" ) {

                if ( !Helper::isScalar( $data ) ) {
                        return "";
                }

                if ( is_bool( $data ) ) {
                        return $data ? 1 : 0;
                }

                if ( !Helper::isNonEmptyStr( $data ) || is_int( $data ) || is_float( $data ) ) {
                        return $data;
                }

                if ( is_bool( $data ) ) {
                        return $data ? 1 : 0;
                }

                $data = Helper::toUtf8( $data );

                $data = $this->escapeSpecialChar( $data );

                return $data;
        }

        
        private function escapeSpecialChar( $data = "" ) {

                if ( Helper::isEmptyStr( $data ) ) {
                        return $data;
                }

                $active_content_triggers = [ '=', '+', '-', '@' ];

                if ( in_array( mb_substr( $data, 0, 1 ), $active_content_triggers, true ) ) {
                        $data = "'" . $data;
                }

                return $data;
        }

        
        private function isEmptyRow( $row = null ) {

                if ( !Helper::isNonEmptyArray( $row ) || Helper::isMultidimensional( $row ) ) {
                        return true;
                }
                return strlen( trim( implode( "", $row ) ) ) === 0;
        }

        
        public function clear() {

                if ( is_resource( $this->fileHandle ) ) {
                        fclose( $this->fileHandle );
                }

                unset( $this->fileHandle, $this->delimiter );
        }

}
